package org.eclipse.tm.tcf.testservice.ui.views;


import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ComboBoxCellEditor;
import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.VerifyEvent;
import org.eclipse.swt.events.VerifyListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.tm.tcf.extender.peer.IPeer;
import org.eclipse.tm.tcf.extensions.TCFTCP;
import org.eclipse.tm.tcf.testservice.ITestServiceProxy;
import org.eclipse.tm.tcf.testservice.TestInstruction;
import org.eclipse.tm.tcf.testservice.TestResult;
import org.eclipse.tm.tcf.testservice.TestServiceProxy;
import org.eclipse.tm.tcf.testservice.TestServiceStatus;
import org.eclipse.tm.tcf.testservice.ITestServiceProxy.ITestEventListener;
import org.eclipse.tm.tcf.testservice.ITestServiceProxy.TestDone;
import org.eclipse.tm.tcf.testservice.ui.Activator;
import org.eclipse.tm.tcf.user.TCF;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;


/**
 * This sample class demonstrates how to plug-in a new
 * workbench view. The view shows data obtained from the
 * model. The sample creates a dummy model on the fly,
 * but a real implementation would connect to the model
 * available either in this or another plug-in (e.g. the workspace).
 * The view is connected to the model using a content provider.
 * <p>
 * The view uses a label provider to define how model
 * objects should be presented in the view. Each
 * view can present the same model objects using
 * different labels and icons, if needed. Alternatively,
 * a single label provider can be shared between views
 * in order to ensure that objects of the same type are
 * presented in the same way everywhere.
 * <p>
 */

public class TestServiceView extends ViewPart {
	
	
	private static final String[] COLUMNS_NAMES = new String[]{"id", "operation", "arg1", "arg2", "result", "check", "rem"};
	private static final String[] OPERATIONS_NAMES = new String[]{TestInstruction.ADD_INSTRUCTION, TestInstruction.SUB_INSTRUCTION, TestInstruction.MUL_INSTRUCTION, TestInstruction.DIV_INSTRUCTION, "Unavaible Operation", TestInstruction.INT_ERROR_OPERATION};
	private IPeer peer;
	private ITestServiceProxy proxy;
	private Table table;
	private TableViewer viewer;
	private Action runTestCommand;
	private Action addInstruction;
	private Action removeInstruction;
	private Action connect;
	private Action disconnect;
	private TestEventListener testListener;
	private TestServiceStatus status;
	private final Map<Integer, TestInstruction> instructions = new HashMap<Integer, TestInstruction>();
	private final Map<Integer, TestResult> results = new HashMap<Integer, TestResult>();
	private final Object waitForEvent = new Object();
	

	/*
	 * The content provider class is responsible for
	 * providing objects to the view. It can wrap
	 * existing objects in adapters or simply return
	 * objects as-is. These objects may be sensitive
	 * to the current input of the view, or ignore
	 * it and always show the same content 
	 * (like Task List, for example).
	 */
	 
	class TestEventListener implements ITestEventListener{
		public void testServiceStatus(final TestServiceStatus status) {
			Display.getDefault().asyncExec(new Runnable(){
				public void run() {
					synchronized(waitForEvent){
						TestServiceView.this.status = status;
						runTestCommand.setEnabled(status.getStatus().equals(TestServiceStatus.AVAIABLE_STATE));	
						waitForEvent.notifyAll();
					}			
				}
			});
		}
	}
	class TestContentProvider implements IStructuredContentProvider {
		public void inputChanged(Viewer v, Object oldInput, Object newInput) {
		}
		public void dispose() {
		}
		public Object[] getElements(Object parent) {
			if(parent instanceof  Map){
				return ((Map)parent).values().toArray();
			} else {
				return new Object[0];
			}
		}
	}
	class TestLabelProvider extends LabelProvider implements ITableLabelProvider {
		private  final ImageDescriptor uncheck = ImageDescriptor.createFromURL(Platform.getBundle(Activator.PLUGIN_ID).getResource("/icons/help.png"));
		private  final ImageDescriptor ok = ImageDescriptor.createFromURL(Platform.getBundle(Activator.PLUGIN_ID).getResource("/icons/accept.png"));
		private  final ImageDescriptor ko = ImageDescriptor.createFromURL(Platform.getBundle(Activator.PLUGIN_ID).getResource("/icons/exclamation.png"));
		public String getColumnText(Object obj, int index) {
			String label = "";
			if(obj instanceof TestInstruction){
				TestInstruction instruction = (TestInstruction) obj;
				switch (index) {
				case 0:
					label += instruction.getId();
					break;
				case 1:
					label += instruction.getOperation();
					break;
				case 2:
					label += instruction.getArg1();
					break;
				case 3:
					label += instruction.getArg2();
					break;
				case 4:
					TestResult result4 = results.get(instruction.getId());
					if(result4 != null) label += result4.getResult();
					break;
				case 6:
					TestResult result5 = results.get(instruction.getId());
					if(result5 != null) label += result5.getRem();
					break;
				default:
					label = null;
					break;
				}
			}
			return label;
		}
		public Image getColumnImage(Object obj, int index) {
			Image image = null;
			if(obj instanceof TestInstruction){
				TestInstruction instruction = (TestInstruction) obj;
				if(index == 5){
					if(!results.containsKey(instruction.getId())){
						image = uncheck.createImage();
					} else if(!results.get(instruction.getId()).isChecked()){
						image = uncheck.createImage();
					} else if(results.get(instruction.getId()).isOk()){
						image = ok.createImage();
					} else {
						image = ko.createImage();
					}
				}
			}
			return image;
		}
		public Image getImage(Object obj) {
			return null;
		}
	}
	class TestSorter extends ViewerSorter {
		private int property; 
		public TestSorter(int property){
			super();
			this.property = property;
		}
		public int compare(Viewer viewer, Object e1, Object e2){
			int comp = 0;
			if(e1 instanceof TestInstruction && e2 instanceof TestInstruction){
				TestInstruction i1 = (TestInstruction)e1;
				TestInstruction i2 = (TestInstruction)e2;
				switch (property) {
				case 0:
					comp = i1.getId()-i2.getId();
					break;
				case 1:
					comp = getComparator().compare(i1.getOperation(), i2.getOperation());
					if(comp == 0) comp = i1.getId()-i2.getId();
					break;
				case 2:
					comp = i1.getArg1() - i2.getArg1();
					if(comp == 0) comp = i1.getId()-i2.getId();
					break;
				case 3:
					comp = i1.getArg2() - i2.getArg2();
					if(comp == 0) comp = i1.getId()-i2.getId();
					break;
				case 4:
					TestResult r1 = results.get(i1.getId());
					TestResult r2 = results.get(i2.getId());
					comp = ((r1==null)?0:r1.getResult()) - ((r2==null)?0:r2.getResult());
					if(comp == 0) comp = i1.getId()-i2.getId();
					break;
				case 5:
					int r5 = 0;
					int r6 = 0;
					if(!results.containsKey(i1.getId())){
						r5 = 0;
					} else if(!results.get(i1.getId()).isChecked()){
						r5 = 0;
					} else if(!results.get(i1.getId()).isOk()){
						r5 = 1;
					} else if(results.get(i1.getId()).isOk()){
						r5 = 2;
					}
					if(!results.containsKey(i2.getId())){
						r6 = 0;
					} else if(!results.get(i2.getId()).isChecked()){
						r6 = 0;
					} else if(!results.get(i2.getId()).isOk()){
						r6 = 1;
					} else if(results.get(i2.getId()).isOk()){
						r6 = 2;
					}
					comp = r5 - r6;
					if(comp == 0) comp = i1.getId()-i2.getId();
					break;
				case 6:
					TestResult r3 = results.get(i1.getId());
					TestResult r4 = results.get(i2.getId());
					comp = getComparator().compare(((r3==null)?null:r3.getRem()), ((r4==null)?null:r4.getRem()));
					if(comp == 0) comp = i1.getId()-i2.getId();
					break;
				default:
					comp = super.compare(viewer, e1, e2);
					break;
					
				}
				
			}else{
				comp = super.compare(viewer, e1, e2);
			}
			return comp;
		}
	}
	class TestCellModifier implements ICellModifier {
		public boolean canModify(Object element, String property) {
			boolean value = false;
			int columnIndex = Arrays.asList(COLUMNS_NAMES).indexOf(property);
			switch (columnIndex) {
			case 1:
				value = true;
				break;
			case 2:
				value = true;
				break;
			case 3:
				value = true;
				break;
			default:
				value = false;
				break;
			}
			return value;
		}
		public Object getValue(Object element, String property) {
			Object value = null;
			int columnIndex = Arrays.asList(COLUMNS_NAMES).indexOf(property);
			TestInstruction instruction = (TestInstruction) element;
			switch (columnIndex) {
			case 1:
				value = Arrays.asList(OPERATIONS_NAMES).indexOf(instruction.getOperation());
				break;
			case 2:
				value = new Integer(instruction.getArg1()).toString();
				break;
			case 3:
				value = new Integer(instruction.getArg2()).toString();
						break;
			default:
				value = "";
				break;
			}
			return value;
		}
		public void modify(Object element, String property, Object value) {
			int columnIndex = Arrays.asList(COLUMNS_NAMES).indexOf(property);
			TestInstruction instruction = (TestInstruction)((TableItem) element).getData();
			switch (columnIndex) {
			case 0:
				break;
			case 1:
				Integer val1 = (Integer) value;
				String res1 = OPERATIONS_NAMES[val1];
				instruction.setOperation(res1);
				results.remove(instruction.getId());
				break;
			case 2:
				String val2 = (String) value;
				Integer res2 = (Integer) Integer.decode(val2);
				instruction.setArg1(res2);
				results.remove(instruction.getId());
				break;
			case 3:
				String val3 = (String) value;
				Integer res3 = (Integer) Integer.decode(val3);
				instruction.setArg2(res3);
				results.remove(instruction.getId());
				break;
			default:
				value = null;
				break;
			}
			viewer.refresh();
		}
	}
	class TestVerifyListener implements VerifyListener{
		public void verifyText(VerifyEvent e) {
			e.doit = e.text.matches("[\\-0-9]*");
		}
	}

	/**
	 * The constructor.
	 */
	public TestServiceView() {
	}

	/**
	 * This is a callback that will allow us
	 * to create the viewer and initialize it.
	 */
	public void createPartControl(Composite parent) {
		createViewer(parent);

		// Create the help context id for the viewer's control

		PlatformUI.getWorkbench().getHelpSystem().setHelp(viewer.getControl(), "org.eclipse.tm.tcf.testservice.ui.viewer");
		makeActions();
		hookContextMenu();
		contributeToActionBars();
	}
	
	private void configureTable() {
		
		GridData gridData = new GridData(GridData.FILL_BOTH);
		gridData.grabExcessVerticalSpace = true;
		gridData.horizontalSpan = 3;
		table.setLayoutData(gridData);		
					
		table.setLinesVisible(true);
		table.setHeaderVisible(true);

		// 1st column with id
		TableColumn column = new TableColumn(table, SWT.CENTER, 0);		
		column.setText(COLUMNS_NAMES[0]);
		column.setWidth(40);
		column.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e){
				viewer.setSorter(new TestSorter(0));
			}
		});
		
		// 2nd column with operation
		column = new TableColumn(table, SWT.LEFT, 1);
		column.setText(COLUMNS_NAMES[1]);
		column.setWidth(160);
		
		column.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e){
				viewer.setSorter(new TestSorter(1));
			}
		});

		// 3rd column with arg1
		column = new TableColumn(table, SWT.LEFT, 2);
		column.setText(COLUMNS_NAMES[2]);
		column.setWidth(60);
		column.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e){
				viewer.setSorter(new TestSorter(2));
			}
		});

		// 4th column with arg2
		column = new TableColumn(table, SWT.LEFT, 3);
		column.setText(COLUMNS_NAMES[3]);
		column.setWidth(60);
		column.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e){
				viewer.setSorter(new TestSorter(3));
			}
		});
		
		// 4th column with result 
		column = new TableColumn(table, SWT.LEFT, 4);
		column.setText(COLUMNS_NAMES[4]);
		column.setWidth(60);
		column.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e){
				viewer.setSorter(new TestSorter(4));
			}
		});
		
		// 5th column with check 
		column = new TableColumn(table, SWT.CENTER, 5);
		column.setText(COLUMNS_NAMES[5]);
		column.setWidth(40);
		column.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e){
				viewer.setSorter(new TestSorter(5));
			}
		});
		
		// 6th column with check 
		column = new TableColumn(table, SWT.LEFT, 6);
		column.setText(COLUMNS_NAMES[6]);
		column.setWidth(400);
		column.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e){
				viewer.setSorter(new TestSorter(6));
			}
		});
	}
	
	private void createViewer(Composite parent){
		viewer = new TableViewer(parent, SWT.SINGLE | SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL | 
				SWT.FULL_SELECTION | SWT.HIDE_SELECTION);
		table = viewer.getTable();
		configureTable();
		viewer.setUseHashlookup(true);
		viewer.setColumnProperties(COLUMNS_NAMES);
		
		CellEditor[] editors = new CellEditor[COLUMNS_NAMES.length];
		editors[0] = null;
		editors[1] = new ComboBoxCellEditor(table, OPERATIONS_NAMES, SWT.READ_ONLY);
		TextCellEditor textEditor = new TextCellEditor(table);
		((Text) textEditor.getControl()).addVerifyListener(new TestVerifyListener());
		editors[2] = textEditor;
		textEditor = new TextCellEditor(table);
		((Text) textEditor.getControl()).addVerifyListener(new TestVerifyListener());
		editors[3] = textEditor;
		editors[4] = null;
		editors[5] = null;
		editors[6] = null;
		
		viewer.setCellEditors(editors);
		viewer.setCellModifier(new TestCellModifier());
		viewer.setSorter(new TestSorter(0));
		
		viewer.setContentProvider(new TestContentProvider());
		viewer.setLabelProvider(new TestLabelProvider());
		viewer.setInput(instructions);
	}

	private void hookContextMenu() {
		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				TestServiceView.this.fillContextMenu(manager);
			}
		});
		Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, viewer);
	}

	private void contributeToActionBars() {
		IActionBars bars = getViewSite().getActionBars();
		fillLocalPullDown(bars.getMenuManager());
		fillLocalToolBar(bars.getToolBarManager());
	}

	private void fillLocalPullDown(IMenuManager manager) {
		manager.add(connect);
		manager.add(disconnect);
		manager.add(new Separator());
		manager.add(runTestCommand);
		manager.add(new Separator());
		manager.add(addInstruction);	
		manager.add(removeInstruction);
	}

	private void fillContextMenu(IMenuManager manager) {
		manager.add(connect);
		manager.add(disconnect);
		manager.add(new Separator());
		manager.add(runTestCommand);
		manager.add(new Separator());
		manager.add(addInstruction);	
		manager.add(removeInstruction);
		// Other plug-ins can contribute there actions here
		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
		
	}
	
	private void fillLocalToolBar(IToolBarManager manager) {
		manager.add(connect);
		manager.add(disconnect);
		manager.add(new Separator());
		manager.add(runTestCommand);
		manager.add(new Separator());
		manager.add(addInstruction);	
		manager.add(removeInstruction);
	}

	private void makeActions() {
		connect = new Action() {
			public void run() {
				connectTarget();
			}
		};
		connect.setText("Connect Target");
		connect.setToolTipText("Set connection to the target");
		ImageDescriptor connectImage = ImageDescriptor.createFromURL(Platform.getBundle(Activator.PLUGIN_ID).getResource("/icons/connect.png"));
		connect.setImageDescriptor(connectImage);
		connect.setEnabled(true);
		
		disconnect = new Action() {
			public void run() {
				disconnectTarget();
			}
		};
		disconnect.setText("Disconnect Target");
		disconnect.setToolTipText("Close connection to the target");
		ImageDescriptor disconnectImage = ImageDescriptor.createFromURL(Platform.getBundle(Activator.PLUGIN_ID).getResource("/icons/disconnect.png"));
		disconnect.setImageDescriptor(disconnectImage);
		disconnect.setEnabled(false);
		
		runTestCommand = new Action() {
			public void run() {
				runTest();
			}
		};
		runTestCommand.setText("Run Tests");
		runTestCommand.setToolTipText("Run the tests and return the results");
		ImageDescriptor runImage = ImageDescriptor.createFromURL(Platform.getBundle(Activator.PLUGIN_ID).getResource("/icons/run.gif"));
		runTestCommand.setImageDescriptor(runImage);
		runTestCommand.setEnabled(false);
		
		addInstruction = new Action() {
			public void run(){
				TestInstruction instruction = new TestInstruction(TestInstruction.ADD_INSTRUCTION, 1, 1);
				instructions.put(instruction.getId(), instruction);
				viewer.refresh();
			}
		};
		addInstruction.setText("Add Instruction");
		addInstruction.setToolTipText("Add a new test instruction");
		ImageDescriptor addImage = ImageDescriptor.createFromURL(Platform.getBundle(Activator.PLUGIN_ID).getResource("/icons/add.gif"));
		addInstruction.setImageDescriptor(addImage);
		
		removeInstruction = new Action() {
			public void run(){
				TestInstruction instruction = (TestInstruction) ((IStructuredSelection)viewer.getSelection()).getFirstElement();
				if(instruction != null){
					instructions.remove(instruction.getId());
					viewer.refresh();
				} else {
					Object obj = viewer.getElementAt(0);
					if(obj != null){
						instruction = (TestInstruction)obj;
						instructions.remove(instruction.getId());
						viewer.refresh();
					}
				}
				
			}
		};
		removeInstruction.setText("Remove Instruction");
		removeInstruction.setToolTipText("Remove a test instruction");
		ImageDescriptor removeImage = ImageDescriptor.createFromURL(Platform.getBundle(Activator.PLUGIN_ID).getResource("/icons/remove.gif"));
		removeInstruction.setImageDescriptor(removeImage);
	}
	private void showMessage(final String message) {
		MessageDialog.openInformation(
			viewer.getControl().getShell(),
			"Test Service View",
			message);
	}
	private void showError(final String message){
		MessageDialog.openError(
				viewer.getControl().getShell(),
				"Test Service View",
				message);
	}
	private void runTest(){
		ProgressMonitorDialog pd = new ProgressMonitorDialog(viewer.getControl().getShell());
		try {
			
			pd.run(true, false, new IRunnableWithProgress(){
				
				public void run(final IProgressMonitor monitor)
						throws InvocationTargetException, InterruptedException {
					
					final Object lock = new Object();
					Collection<TestInstruction> todoInstructions = new ArrayList<TestInstruction>();
					for(TestInstruction instruction : instructions.values()){
						if(!results.containsKey(instruction.getId())) todoInstructions.add(instruction);
					}
					monitor.beginTask("Run Tests Task", todoInstructions.size());
					TestInstruction[] array = todoInstructions.toArray(new TestInstruction[todoInstructions.size()]);
					proxy.testRun(array, new TestDone(){

						public void testProgres(String token,
								final TestResult result) {
							Display.getDefault().asyncExec(new Runnable(){
								public void run() {
									monitor.worked(1);
									results.put(result.getId(), result);
									viewer.refresh();
								}
							});
						}

						public void testResult(final String token) {
							Display.getDefault().asyncExec(new Runnable(){
								public void run() {
									monitor.done();			
									synchronized(lock){
										lock.notify();
									}
								}
							});
						}

						public void testterminated(final String token, final Exception error) {
							Display.getDefault().asyncExec(new Runnable(){
								public void run() {
									showError(error.getMessage());
									monitor.done();
									synchronized(lock){
										lock.notify();
									}
								}
							});
						}
						
					});
					synchronized(lock){
						lock.wait();
					}
				}
				
			});
			
		} catch (Exception e) {
			showError(e.getMessage());
			e.printStackTrace();
		}
	}
	private void connectTarget(){
		InputDialog dialog = new InputDialog(
				viewer.getControl().getShell(), 
				"Connect to target",
				"Enter target adress",
				"127.0.0.1:1534",
				null);
		dialog.setBlockOnOpen(true);
		dialog.open();
		int code = dialog.getReturnCode();
		if(code == dialog.OK){
			final String value = dialog.getValue();
			final String[] values = value.split(":");
			final String address = values[0];
			final String port = values[1];
			final Map<String, String> attrs = new HashMap<String, String>();
			attrs.put(TCFTCP.HOST_ATTR, address);
			attrs.put(TCFTCP.PORT_ATTR, port);
			ProgressMonitorDialog pd = new ProgressMonitorDialog(viewer.getControl().getShell());
			try {
				pd.run(true, false, new IRunnableWithProgress(){
					public void run(final IProgressMonitor monitor)
							throws InvocationTargetException, InterruptedException {
						monitor.beginTask("Create peer", 4);
						try {
							peer = TCF.createPeer(value, TCFTCP.TCP_TYPE, attrs);
						} catch (Exception e) {
							showError(e.getMessage());
							monitor.done();
							e.printStackTrace();
							return;
						}
						monitor.worked(1);
						monitor.setTaskName("Connect to target");
						try {
							TCF.connect(peer);
							while(!TCF.isConnected(peer)){
								Thread.sleep(50);
							}
						} catch (Exception e) {
							showError(e.getMessage());
							monitor.done();
							e.printStackTrace();
							return;
						}
						monitor.worked(1);
						monitor.setTaskName("Get service");
						try {
							proxy = (ITestServiceProxy) TCF.getServiceProxy(peer, TestServiceProxy.NAME);
						} catch (Exception e) {
							showError(e.getMessage());
							monitor.done();
							e.printStackTrace();
							return;
						}
						monitor.worked(1);
						monitor.setTaskName("Set listener");
						try{
							testListener = new TestEventListener();
							proxy.addTestEventListener(testListener);
						} catch (Exception e) {
							showError(e.getMessage());
							monitor.done();
							e.printStackTrace();
							return;
						}
						monitor.worked(1);
						monitor.setTaskName("Get service status");
						try {
							proxy.getTestServiceStatus();
							synchronized(waitForEvent){
								waitForEvent.wait();
							}
						} catch (Exception e) {
							showError(e.getMessage());
							monitor.done();
							e.printStackTrace();
							return;
						}
						monitor.worked(1);
						disconnect.setEnabled(true);
						connect.setEnabled(false);
						monitor.done();
					}
				});
			} catch (Exception e) {
				showError(e.getMessage());
				e.printStackTrace();
			}
		}
	}
	private void disconnectTarget(){
		ProgressMonitorDialog pd = new ProgressMonitorDialog(viewer.getControl().getShell());
		try {
			pd.run(true, false, new IRunnableWithProgress(){
				public void run(final IProgressMonitor monitor)
						throws InvocationTargetException, InterruptedException {
					monitor.beginTask("Remove listener", 3);
					try{
						proxy.removeTestEventListener(testListener);
						testListener = null;
					} catch (Exception e) {
						showError(e.getMessage());
						monitor.done();
						e.printStackTrace();
						return;
					}
					monitor.worked(1);
					monitor.setTaskName("Remove Service");
					try{
						proxy = null;
					} catch (Exception e) {
						showError(e.getMessage());
						monitor.done();
						e.printStackTrace();
						return;
					}
					monitor.worked(1);
					monitor.setTaskName("disconnect");
					try {
						TCF.disconnect(peer);
					} catch (Exception e) {
						showError(e.getMessage());
						monitor.done();
						e.printStackTrace();
						return;
					}
					monitor.worked(1);
					disconnect.setEnabled(false);
					connect.setEnabled(true);
					runTestCommand.setEnabled(false);
					monitor.done();
				}
			});
		} catch (Exception e) {
			showError(e.getMessage());
			e.printStackTrace();
		}
	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus() {
		viewer.getControl().setFocus();
	}
	
	public void dispose(){
		if(disconnect.isEnabled()){
			proxy.removeTestEventListener(testListener);
			TCF.disconnect(peer);
		}
		super.dispose();
	}
}