package org.eclipse.tm.tcf.testservice;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Map;

import org.eclipse.tm.tcf.extender.service.proxy.IServiceProxyInterface;
import org.eclipse.tm.tcf.extender.service.proxy.IServiceProxyInterface.ICommandListener;
import org.eclipse.tm.tcf.extender.service.proxy.IServiceProxyInterface.IEventListener;

public class TestServiceProxy implements ITestServiceProxy, IEventListener {
	
	private final IServiceProxyInterface proxyInterface;
	private final Collection<ITestEventListener> listeners = new ArrayList<ITestEventListener>();
	
	public TestServiceProxy(IServiceProxyInterface proxyInterface){
		super();
		this.proxyInterface = proxyInterface;
		proxyInterface.addEventListener(NAME, this);
	}

	public void addTestEventListener(ITestEventListener listener) {
		listeners.add(listener);
	}

	public String getTestServiceStatus() {
		try {
			return proxyInterface.sendCommand(NAME, GET_STATUS_COMMAND, new Object[0], null);
		} catch (Exception e) {
			return null;		
		}
	}

	public void removeTestEventListener(ITestEventListener listener) {
		listeners.remove(listener);
	}

	@SuppressWarnings("unchecked")
	public String testRun(final TestInstruction[] instructions, final TestDone done) {
		try {
			Map[] data = new Map[instructions.length];
			for(int i=0; i<data.length; i++){
				data[i] = instructions[i].toMap();
			}
			return proxyInterface.sendCommand(NAME, RUN_TEST_COMMAND, data, new ICommandListener(){

				public void progress(String token, Object[] data) {
					TestResult result = new TestResult((Map) data[0]);
					done.testProgres(token, result);
				}

				public void result(String token, Object[] data) {
					if(data == null || data.length == 0){
						done.testResult(token);
					}
					else if(data[0] instanceof String && data[0].equals(ERROR_TAG)){
						done.testterminated(token, new Exception((String)data[1]));
					}
				}

				public void terminated(String token, Exception error) {
					done.testterminated(token, error);
				}
				
			});
		} catch (Exception e) {
			done.testterminated(null, e);
			return null;
		}
	}

	public String getName() {
		return NAME;
	}

	@SuppressWarnings("unchecked")
	public void event(String name, Object[] data) {
		if(name.equals(STATUS_EVENT)){
			TestServiceStatus status = new TestServiceStatus((Map)data[0]);
			for(ITestEventListener listener : listeners){
				listener.testServiceStatus(status);
			}
		}
		
	}
	
	public void terminate(){
		proxyInterface.removeEventListener(NAME, this);
	}

}
