package org.eclipse.tm.tcf.testservice;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.tm.tcf.extender.peer.IPeer;
import org.eclipse.tm.tcf.extender.service.IServiceUtilities;
import org.eclipse.tm.tcf.extender.service.ITaskUtilities;
import org.eclipse.tm.tcf.extender.service.provider.IServiceProviderInterface;
import org.eclipse.tm.tcf.extender.service.provider.IServiceProviderInterface.ICommandServer;
import org.eclipse.tm.tcf.extender.service.proxy.IServiceProxyInterface;
import org.eclipse.tm.tcf.extender.service.proxy.IServiceProxyInterface.ICommandListener;
import org.eclipse.tm.tcf.extender.service.valueadder.IServiceValueAdderManager;
import org.eclipse.tm.tcf.extensions.TCFTCP;

public class TestServiceValueAdderManager implements IServiceValueAdderManager {
	
	private final Map<IPeer, TestServiceValueAdder> valueAdders = new HashMap<IPeer, TestServiceValueAdder>();
	private ITaskUtilities taskUtilities;

	public void createServiceValueAdder(IPeer peer,
			IServiceProxyInterface proxyInterface,
			IServiceProviderInterface providerInterface) {
		TestServiceValueAdder valueAdder = new TestServiceValueAdder(proxyInterface, providerInterface);
		valueAdders.put(peer, valueAdder);

	}

	public String getType() {
		return TCFTCP.TCP_TYPE;
	}

	public void removeServiceValueAdder(IPeer peer) {
		TestServiceValueAdder valueAdder = valueAdders.remove(peer);
		valueAdder.terminate();
	}

	public void setServiceUtilities(IServiceUtilities utilities) {
		// TODO Auto-generated method stub

	}
	
	public void setTaskUtilities(ITaskUtilities utilities) {
		taskUtilities = utilities;
	}
	
	private class TestServiceValueAdder implements ICommandServer, ICommandListener {
		
		private IServiceProxyInterface proxyInterface;
		private IServiceProviderInterface providerInterface;
		
		private final Map<String, String> tokensTranslation = new HashMap<String, String>();
		private final Collection<String> tokens = new ArrayList<String>();
		private final Map<Integer, TestInstruction> instructions = new HashMap<Integer, TestInstruction>(); 
		
		public TestServiceValueAdder(IServiceProxyInterface proxyInterface,
			IServiceProviderInterface providerInterface){
			this.proxyInterface = proxyInterface;
			this.providerInterface = providerInterface;
			providerInterface.addCommandServer(ITestService.NAME, this);
		}
		
		public void terminate(){
			providerInterface.removeCommandServer(ITestService.NAME, this);
		}

		public void command(String token, String name, Object[] data) {
			String targetToken = proxyInterface.sendCommand(ITestService.NAME, name, data, this);
			tokensTranslation.put(targetToken, token);
			if(name.equals(ITestService.RUN_TEST_COMMAND)){
				tokens.add(targetToken);
				for(Object obj : data){
					TestInstruction instruction = new TestInstruction((Map) obj);
					instructions.put(instruction.getId(), instruction);
				}
			}
		}

		public void progress(String token, Object[] data) {
			if(!tokensTranslation.containsKey(token)) return;
			if(tokens.contains(token)){
				TestResult result = new TestResult((Map)data[0]);
				if(!instructions.containsKey(result.getId())){
					result.check(true);
					result.ok(false);
					result.setRem("Internal Error Instruction Id not found");
				} else if (result.isChecked()) {
				} else {
					try {
						int res = run(instructions.get(result.getId()));
						if(res != result.getResult()){
							result.check(true);
							result.ok(false);
							result.setRem("False result, it should be : "+res);
						} else {
							result.check(true);
							result.ok(true);
						}
					} catch (Exception e) {
						result.check(true);
						result.ok(false);
						result.setRem(e.getMessage());
					}
				}
				data = new Object[]{result.toMap()};
			}
			providerInterface.sendProgress(tokensTranslation.get(token), data);
		}

		public void result(String token, Object[] data) {
			if(!tokensTranslation.containsKey(token)) return;
			providerInterface.sendResult(tokensTranslation.get(token), data);
			if(tokens.contains(token)) tokens.remove(token);
			tokensTranslation.remove(token);
		}

		public void terminated(String token, Exception error) {
			if(!tokensTranslation.containsKey(token)) return;
			if(tokens.contains(token)){
				providerInterface.sendResult(tokensTranslation.get(token), new String[]{ITestService.ERROR_TAG, error.getMessage()});
				tokens.remove(token);
			}
			tokensTranslation.remove(token);
		}
		
		private int run(TestInstruction instruction) throws Exception{
			int result = 0;
			if(instruction.getOperation().equals(TestInstruction.ADD_INSTRUCTION)) result = instruction.getArg1() + instruction.getArg2();
			else if(instruction.getOperation().equals(TestInstruction.SUB_INSTRUCTION)) result = instruction.getArg1() - instruction.getArg2();
			else if(instruction.getOperation().equals(TestInstruction.MUL_INSTRUCTION)) result = instruction.getArg1() * instruction.getArg2();
			else if(instruction.getOperation().equals(TestInstruction.DIV_INSTRUCTION)) result = instruction.getArg1() / instruction.getArg2();
			else throw new Exception("Unsuported operation in Check : "+ instruction.getOperation());
			return result;
		}
		
	}

}
