package org.eclipse.tm.tcf.extender.service.provider;

public interface IServiceProviderInterface {

	void sendEvent(String serviceId, String name, Object[] data);

	void sendResult(String token, Object[] data);

	void sendProgress(String token, Object[] data);

	/**
	 * Subscribe a command server. The server will be notified about command
	 * messages received through this channel for given service.
	 * 
	 * @param service
	 *            - local service implementation
	 * @param server
	 *            - implementation of service commands listener
	 */
	void addCommandServer(String serviceId, ICommandServer server);

	/**
	 * Remove a command server.
	 * 
	 * @param service
	 *            - local service implementation
	 * @param server
	 *            - implementation of service commands listener
	 */
	void removeCommandServer(String serviceId, ICommandServer server);

	interface ICommandServer {

		/**
		 * Called every time a command is received from remote peer.
		 * 
		 * @param token
		 *            - command handle
		 * @param name
		 *            - command name
		 * @param data
		 *            - command arguments encoded into array of bytes
		 */
		void command(String token, String name, Object[] data);

	}

}
