package org.eclipse.tm.tcf.extender.service.proxy;

public interface IServiceProxyInterface {

	String sendCommand(String serviceId, String commandName, Object[] data,
			ICommandListener commandListener);

	interface ICommandListener {
		/**
		 * Called when progress message (intermediate result) is received from
		 * remote peer.
		 * 
		 * @param token
		 *            - command handle
		 * @param data
		 *            - progress message arguments encoded into array of bytes
		 */
		void progress(String token, Object[] data);

		/**
		 * Called when command result received from remote peer.
		 * 
		 * @param token
		 *            - command handle
		 * @param data
		 *            - command result message arguments encoded into array of
		 *            bytes
		 */
		void result(String token, Object[] data);

		/**
		 * Called when communication channel was closed while command was
		 * waiting for result.
		 * 
		 * @param token
		 *            - command handle
		 * @param error
		 *            - exception that forced the channel to close
		 */
		void terminated(String token, Exception error);
	}

	void addEventListener(String serviceId, IEventListener eventListener);

	void removeEventListener(String serviceId, IEventListener eventListener);

	interface IEventListener {
		/**
		 * Called when service event message is received
		 * 
		 * @param name
		 *            - event name
		 * @param data
		 *            - event arguments encode as array of bytes
		 */
		void event(String name, Object[] data);

	}

}
