package org.eclipse.tm.tcf.extensions.channel;

import java.io.IOException;

import org.eclipse.tm.tcf.extender.channel.IChannelInterface;
import org.eclipse.tm.tcf.extender.message.Command;
import org.eclipse.tm.tcf.extender.message.EndOfStream;
import org.eclipse.tm.tcf.extender.message.Event;
import org.eclipse.tm.tcf.extender.message.IMessage;
import org.eclipse.tm.tcf.extender.message.Progress;
import org.eclipse.tm.tcf.extender.message.Result;
import org.eclipse.tm.tcf.extender.peer.IPeer;
import org.eclipse.tm.tcf.extender.service.ITaskUtilities;
import org.eclipse.tm.tcf.extender.util.JSON;

public abstract class AbstractTCFChannel extends AbstractChannel {

	public static final int BUFFER_SIZE = 1024;
	public static final int END_OF_MESSAGE = -2;
	public static final int END_OF_STREAM = -1;
	public static final int END_OF_STRING = 0;
	public static final String ENCODING = "UTF8";

	public AbstractTCFChannel(IPeer peer, IChannelInterface channelInterface,
			ITaskUtilities taskUtilities) {
		super(peer, channelInterface, taskUtilities);
	}

	public void initRead() throws Exception {

	}

	public void initWrite() throws Exception {

	}

	public void terminateRead() throws Exception {

	}

	public void terminateWrite() throws Exception {

	}

	public IMessage readMessage() throws Exception {
		IMessage message = null;
		String token = "";
		String service = "";
		String name = "";
		Object[] data = new Object[0];
		int readByte = 0;
		while (true) {
			readByte = read();
			switch (readByte) {
			case Command.TYPE:
				readString();
				token = readString();
				service = readString();
				name = readString();
				data = readData();
				message = new Command(token, service, name, data);
				break;
			case Result.TYPE:
				readString();
				token = readString();
				data = readData();
				message = new Result(token, service, name, data);
				break;
			case Progress.TYPE:
				readString();
				token = readString();
				data = readData();
				message = new Progress(token, service, name, data);
				break;
			case Event.TYPE:
				readString();
				service = readString();
				name = readString();
				data = readData();
				message = new Event(token, service, name, data);
				break;
			case END_OF_STREAM:
				message = new EndOfStream();
				break;
			default:
				continue;
			}
			break;
		}
		return message;
	}

	public void writeMessage(IMessage message) throws Exception {
		switch (message.getType()) {
		case Command.TYPE:
			write((int) message.getType());
			write(END_OF_STRING);
			writeString(message.getToken());
			write(END_OF_STRING);
			writeString(message.getService());
			write(END_OF_STRING);
			writeString(message.getName());
			write(END_OF_STRING);
			writeData(message.getData());
			write(END_OF_MESSAGE);
			flush();
			break;
		case Result.TYPE:
			write((int) message.getType());
			write(END_OF_STRING);
			writeString(message.getToken());
			write(END_OF_STRING);
			writeData(message.getData());
			write(END_OF_MESSAGE);
			flush();
			break;
		case Progress.TYPE:
			write((int) message.getType());
			write(END_OF_STRING);
			writeString(message.getToken());
			write(END_OF_STRING);
			writeData(message.getData());
			write(END_OF_MESSAGE);
			flush();
			break;
		case Event.TYPE:
			write((int) message.getType());
			write(END_OF_STRING);
			writeString(message.getService());
			write(END_OF_STRING);
			writeString(message.getName());
			write(END_OF_STRING);
			writeData(message.getData());
			write(END_OF_MESSAGE);
			flush();
			break;
		case END_OF_STREAM:
			write(END_OF_STREAM);
			flush();
			break;
		default:
			break;

		}

	}

	private String readString() throws IOException {
		byte[] bytes = readBytes(END_OF_STRING);
		return new String(bytes, ENCODING);
	}

	private Object[] readData() throws IOException {
		byte[] bytes = readBytes(END_OF_MESSAGE);
		return JSON.parseSequence(bytes);
	}

	private byte[] readBytes(int endByte) throws IOException {
		int readByte = 0;
		int length = 0;
		byte[] buffer = new byte[BUFFER_SIZE];
		while (true) {
			readByte = read();
			if (readByte == endByte)
				break;
			if (length >= buffer.length) {
				byte[] temp = new byte[buffer.length * 2];
				System.arraycopy(buffer, 0, temp, 0, length);
				buffer = temp;
			}
			buffer[length++] = (byte) readByte;
		}
		byte[] result = new byte[length];
		System.arraycopy(buffer, 0, result, 0, length);
		return result;

	}

	private void writeString(String string) throws IOException {
		byte[] bytes = string.getBytes(ENCODING);
		writeBytes(bytes);
	}

	private void writeData(Object[] data) throws IOException {
		byte[] bytes = JSON.toJSONSequence(data);
		writeBytes(bytes);
	}

	private void writeBytes(byte[] bytes) throws IOException {
		if (bytes == null || bytes.length == 0)
			return;
		for (int i = 0; i < bytes.length; i++) {
			write((int) bytes[i]);
		}
	}

	public abstract int read() throws IOException;

	public abstract void write(int n) throws IOException;

	public abstract void flush() throws IOException;
}
