package org.eclipse.tm.tcf.extensions.channel;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.tm.tcf.extender.channel.IChannelInterface;
import org.eclipse.tm.tcf.extender.channel.IChannelManager;
import org.eclipse.tm.tcf.extender.peer.IPeer;
import org.eclipse.tm.tcf.extender.service.ITaskUtilities;
import org.eclipse.tm.tcf.extensions.TCFTCP;
import org.eclipse.tm.tcf.user.TCF;

public class TCPChannelManager implements IChannelManager, Runnable {

	final private Map<String, Socket> servedPeer = new HashMap<String, Socket>();
	final private Map<IPeer, TCPChannel> channels = new HashMap<IPeer, TCPChannel>();
	private ITaskUtilities utilities = null;
	private Thread thread;
	private int port;

	public String getType() {
		return TCFTCP.TCP_TYPE;
	}

	public void run() {
		try {
			ServerSocket server = new ServerSocket(port);
			while (true) {
				final Socket socket = server.accept();
				utilities.invokeLater(new Runnable() {
					public void run() {
						handleSocket(socket);
					}
				});

			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	private void handleSocket(Socket socket) {
		assert utilities.isDispatchThread();
		Map<String, String> attrs = new HashMap<String, String>();
		String hostAttr = socket.getInetAddress().toString();
		String portAttr = Integer.toString(socket.getPort());
		attrs.put(TCFTCP.HOST_ATTR, hostAttr);
		attrs.put(TCFTCP.PORT_ATTR, portAttr);
		String name = hostAttr + ":" + portAttr;
		servedPeer.put(name, socket);
		IPeer peer = null;
		try {
			peer = TCF.createPeer(name, TCFTCP.TCP_TYPE, attrs);
			TCF.connect(peer);
		} catch (Exception e) {
			utilities.error(e);
		}
	}

	public void createChannel(IPeer peer, IChannelInterface channelInterface) {
		assert utilities.isDispatchThread();
		try {
			Socket socket = servedPeer.get(peer.getName());
			if (socket == null) {
				channels.put(peer, new TCPChannel(peer, channelInterface,
						utilities));
			} else {
				channels.put(peer, new TCPChannel(peer, channelInterface,
						utilities, socket));
				servedPeer.remove(peer);
			}
		} catch (final Exception e) {
			utilities.invokeLater(new Runnable() {
				public void run() {
					utilities.error(e);
				}
			});
		}
	}

	public void removeChannel(IPeer peer) {
		assert utilities.isDispatchThread();
		TCPChannel channel = channels.remove(peer);
		try {
			channel.stop();
		} catch (Exception e) {
			utilities.error(e);
		}
	}

	public void setTaskUtilities(ITaskUtilities utilities) {
		this.utilities = utilities;
	}

	public void actAsComServer(Map<String, String> serverConfigurations) {
		thread = new Thread(this);
		thread.setDaemon(true);
		thread.setName("TCF : TCP Server");
		port = Integer.decode(serverConfigurations.get(TCFTCP.PORT_ATTR));
		if (serverConfigurations.get(TCFTCP.PORT_ATTR) == null)
			port = TCFTCP.TCF_PORT;
		thread.start();
	}

}
