package org.eclipse.tm.tcf.internal;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.tm.tcf.extender.peer.IPeer;
import org.eclipse.tm.tcf.extender.service.IServiceUtilities;
import org.eclipse.tm.tcf.extender.service.provider.IServiceProvider;
import org.eclipse.tm.tcf.extender.service.proxy.IServiceProxy;

public class CommunicationManager implements IServiceUtilities {

	private final Map<IPeer, PeerCommunication> coms = new HashMap<IPeer, PeerCommunication>();
	private final Map<IPeer, Map<String, IServiceProxy>> proxies = new HashMap<IPeer, Map<String, IServiceProxy>>();
	private final Map<IPeer, Map<String, IServiceProvider>> providers = new HashMap<IPeer, Map<String, IServiceProvider>>();

	public CommunicationManager() {
		super();
	}

	public boolean isConnected(IPeer peer) {
		assert TCFManager.getTaskManager().isDispatchThread();
		return (coms.get(peer) != null);
	}

	public void connect(IPeer peer) {
		assert TCFManager.getTaskManager().isDispatchThread();
		if (isConnected(peer))
			return;
		proxies.put(peer, new HashMap<String, IServiceProxy>());
		providers.put(peer, new HashMap<String, IServiceProvider>());
		coms.put(peer, new PeerCommunication(peer));
	}

	public void disconnect(IPeer peer) {
		assert TCFManager.getTaskManager().isDispatchThread();
		if (!isConnected(peer))
			return;
		coms.remove(peer).terminate();
	}

	public IServiceProxy getServiceProxy(IPeer peer, String serviceName) {
		assert TCFManager.getTaskManager().isDispatchThread();
		IServiceProxy proxy = null;
		if (proxies.containsKey(peer)) {
			proxy = proxies.get(peer).get(serviceName);
		}
		return proxy;
	}

	public Collection<IServiceProxy> getServiceProxies(IPeer peer) {
		assert TCFManager.getTaskManager().isDispatchThread();
		Collection<IServiceProxy> proxiesCollection = new ArrayList<IServiceProxy>();
		if (proxies.containsKey(peer)) {
			proxiesCollection = new ArrayList<IServiceProxy>(proxies.get(peer)
					.values());
		}
		return proxiesCollection;
	}

	public IServiceProvider getServiceProvider(IPeer peer, String serviceName) {
		assert TCFManager.getTaskManager().isDispatchThread();
		IServiceProvider provider = null;
		if (proxies.containsKey(peer)) {
			provider = providers.get(peer).get(serviceName);
		}
		return provider;
	}

	public Collection<IServiceProvider> getServiceProviders(IPeer peer) {
		assert TCFManager.getTaskManager().isDispatchThread();
		Collection<IServiceProvider> providersCollection = new ArrayList<IServiceProvider>();
		if (providers.containsKey(peer)) {
			providersCollection = new ArrayList<IServiceProvider>(providers
					.get(peer).values());
		}
		return providersCollection;
	}

	/* SERVICE UTILITIES */

	public void publishServiceProvider(IPeer peer, IServiceProvider provider) {
		assert TCFManager.getTaskManager().isDispatchThread();
		if (providers.containsKey(peer)) {
			providers.get(peer).put(provider.getName(), provider);
		}
	}

	public void publishServideProxy(IPeer peer, IServiceProxy proxy) {
		assert TCFManager.getTaskManager().isDispatchThread();
		if (proxies.containsKey(peer)) {
			proxies.get(peer).put(proxy.getName(), proxy);
		}
	}

	public void removeServiceProvider(IPeer peer, String serviceName) {
		assert TCFManager.getTaskManager().isDispatchThread();
		if (providers.containsKey(peer)) {
			providers.get(peer).remove(serviceName);
			if (providers.get(peer).isEmpty() && !isConnected(peer))
				providers.remove(peer);
		}
	}

	public void removeServiceProxy(IPeer peer, String serviceName) {
		assert TCFManager.getTaskManager().isDispatchThread();
		if (proxies.containsKey(peer)) {
			proxies.get(peer).remove(serviceName);
			if (proxies.get(peer).isEmpty() && !isConnected(peer))
				proxies.remove(peer);
		}
	}

}
