package org.eclipse.tm.tcf.internal;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.Platform;
import org.eclipse.tm.tcf.Activator;
import org.eclipse.tm.tcf.extender.channel.IChannelManager;
import org.eclipse.tm.tcf.extender.peer.IPeerManager;
import org.eclipse.tm.tcf.extender.service.provider.IServiceProviderManager;
import org.eclipse.tm.tcf.extender.service.proxy.IServiceProxyManager;
import org.eclipse.tm.tcf.extender.service.valueadder.IServiceValueAdderManager;

public class TCFManager {

	private static final String EXTPT_TCF = "tcf";
	private static final String ELEM_PEER_MANAGER_NAME = "peerManager";
	private static final String ELEM_CHANNEL_MANAGER_NAME = "channelManager";
	private static final String ELEM_SERVICE_PROXY_MANAGER_NAME = "serviceProxyManager";
	private static final String ELEM_SERVICE_PROVIDER_MANAGER_NAME = "serviceProviderManager";
	private static final String ELEM_SERVICE_VALUEADDER__MANAGERNAME = "serviceValueAdderManager";
	// private static final String ATTR_NAME = "name";
	// private static final String ATTR_ID = "id";
	private static final String ATTR_CLASS = "class";

	private static Map<String, IPeerManager> peerManagers;
	private static Map<String, IChannelManager> channelManagers;
	private static Map<String, Collection<IServiceProxyManager>> serviceProxyManagers;
	private static Map<String, Collection<IServiceProviderManager>> serviceProviderManagers;
	private static Map<String, Collection<IServiceValueAdderManager>> serviceValueAdderManagers;

	private static TaskManager taskManager;
	private static CommunicationManager communicationManager;

	private static boolean activated = false;

	private static void init() {

		taskManager = new TaskManager();
		communicationManager = new CommunicationManager();

		peerManagers = new HashMap<String, IPeerManager>();
		channelManagers = new HashMap<String, IChannelManager>();
		serviceProxyManagers = new HashMap<String, Collection<IServiceProxyManager>>();
		serviceProviderManagers = new HashMap<String, Collection<IServiceProviderManager>>();
		serviceValueAdderManagers = new HashMap<String, Collection<IServiceValueAdderManager>>();

		IExtension[] extensions = Platform.getExtensionRegistry()
				.getExtensionPoint(Activator.PLUGIN_ID, EXTPT_TCF)
				.getExtensions();
		IConfigurationElement[] elements;
		String name;
		Object obj;
		for (IExtension extension : extensions) {
			elements = extension.getConfigurationElements();
			for (IConfigurationElement element : elements) {
				name = element.getName();
				try {
					obj = element.createExecutableExtension(ATTR_CLASS);
				} catch (CoreException e) {
					continue;
				}
				if (name.equals(ELEM_PEER_MANAGER_NAME)) {
					if (!(obj instanceof IPeerManager))
						continue;
					IPeerManager manager = (IPeerManager) obj;
					manager.setTaskUtilities(taskManager);
					peerManagers.put(manager.geType(), manager);
				} else if (name.equals(ELEM_CHANNEL_MANAGER_NAME)) {
					if (!(obj instanceof IChannelManager))
						continue;
					IChannelManager manager = (IChannelManager) obj;
					manager.setTaskUtilities(taskManager);
					channelManagers.put(manager.getType(), manager);
				} else if (name.equals(ELEM_SERVICE_PROXY_MANAGER_NAME)) {
					if (!(obj instanceof IServiceProxyManager))
						continue;
					IServiceProxyManager manager = (IServiceProxyManager) obj;
					manager.setTaskUtilities(taskManager);
					manager.setServiceUtilities(communicationManager);
					Collection<IServiceProxyManager> managers = serviceProxyManagers
							.get(manager.getType());
					if (managers == null)
						managers = new ArrayList<IServiceProxyManager>();
					managers.add(manager);
					serviceProxyManagers.put(manager.getType(), managers);
				} else if (name.equals(ELEM_SERVICE_PROVIDER_MANAGER_NAME)) {
					if (!(obj instanceof IServiceProviderManager))
						continue;
					IServiceProviderManager manager = (IServiceProviderManager) obj;
					manager.setTaskUtilities(taskManager);
					manager.setServiceUtilities(communicationManager);
					Collection<IServiceProviderManager> managers = serviceProviderManagers
							.get(manager.getType());
					if (managers == null)
						managers = new ArrayList<IServiceProviderManager>();
					managers.add(manager);
					serviceProviderManagers.put(manager.getType(), managers);
				} else if (name.equals(ELEM_SERVICE_VALUEADDER__MANAGERNAME)) {
					if (!(obj instanceof IServiceValueAdderManager))
						continue;
					IServiceValueAdderManager manager = (IServiceValueAdderManager) obj;
					manager.setTaskUtilities(taskManager);
					manager.setServiceUtilities(communicationManager);
					Collection<IServiceValueAdderManager> managers = serviceValueAdderManagers
							.get(manager.getType());
					if (managers == null)
						managers = new ArrayList<IServiceValueAdderManager>();
					managers.add(manager);
					serviceValueAdderManagers.put(manager.getType(), managers);
				}
			}
		}

		activated = true;
	}

	public static Map<String, IPeerManager> getPeerManagers() {
		if (!activated)
			init();
		return peerManagers;
	}

	public static Map<String, IChannelManager> getChannelManagers() {
		if (!activated)
			init();
		return channelManagers;
	}

	public static Map<String, Collection<IServiceProxyManager>> getServiceProxyManagers() {
		if (!activated)
			init();
		return serviceProxyManagers;
	}

	public static Map<String, Collection<IServiceProviderManager>> getServiceProviderManagers() {
		if (!activated)
			init();
		return serviceProviderManagers;
	}

	public static Map<String, Collection<IServiceValueAdderManager>> getServiceValueAdderManagers() {
		if (!activated)
			init();
		return serviceValueAdderManagers;
	}

	public static TaskManager getTaskManager() {
		if (!activated)
			init();
		return taskManager;
	}

	public static CommunicationManager getCommunicationManager() {
		if (!activated)
			init();
		return communicationManager;
	}

	public static boolean isActivated() {
		return activated;
	}

}
