package org.eclipse.tm.tcf.internal;

import java.util.LinkedList;

import org.eclipse.tm.tcf.extender.service.ITaskUtilities;

public class TaskManager implements Runnable, ITaskUtilities {

	private final LinkedList<Runnable> queue = new LinkedList<Runnable>();
	private final Thread thread;
	private boolean waiting;

	public TaskManager() {
		super();
		thread = new Thread(this);
		thread.setDaemon(true);
		thread.setName("TCF Tasks Dispatch");
		thread.start();
	}

	public synchronized void error(Throwable x) {
		// TODO implement error management
	}

	public synchronized void log(int severity, String message) {
		// TODO implement log management
	}

	public boolean isDispatchThread() {
		return Thread.currentThread() == thread;
	}

	public void run() {
		assert isDispatchThread();
		Runnable task;
		while (true) {
			try {
				task = null;
				synchronized (this) {
					while (queue.isEmpty()) {
						waiting = true;
						wait();
					}
					task = queue.removeFirst();
				}
				task.run();
			} catch (Throwable x) {
				error(x);
			}
		}
	}

	public synchronized void invokeLater(final Runnable task) {
		queue.add(task);
		if (waiting) {
			waiting = false;
			notifyAll();
		}
	}

	public Object invokeAndWaitReturn(final int waitingTime,
			final ReturnRunnable runnable) {
		assert !isDispatchThread();
		final Object[] lock = new Object[1];
		lock[0] = null;
		synchronized (lock) {
			TCFManager.getTaskManager().invokeLater(new Runnable() {
				public void run() {
					try {
						lock[0] = runnable.runAndReturn();
					} catch (Exception e) {
						error(e);
					} finally {
						sync(lock);
					}
				}
			});
			try {
				lock.wait(waitingTime);
			} catch (final Exception e) {
				invokeLater(new Runnable() {
					public void run() {
						error(e);
					}
				});
			}
		}
		return lock[0];
	}

	private static void sync(Object lock) {
		synchronized (lock) {
			lock.notify();
		}
	}

}
