package org.eclipse.tm.tcf.internal;

import java.util.Map;

import org.eclipse.tm.tcf.extender.channel.IChannelInterface;
import org.eclipse.tm.tcf.extender.channel.IChannelManager;
import org.eclipse.tm.tcf.extender.channel.IChannelInterface.IMessageSender;
import org.eclipse.tm.tcf.extender.message.Command;
import org.eclipse.tm.tcf.extender.message.Event;
import org.eclipse.tm.tcf.extender.message.IMessage;
import org.eclipse.tm.tcf.extender.message.Progress;
import org.eclipse.tm.tcf.extender.message.Result;
import org.eclipse.tm.tcf.extender.peer.IPeer;
import org.eclipse.tm.tcf.extender.service.ITaskUtilities;
import org.eclipse.tm.tcf.extender.service.valueadder.IServiceValueAdderManager;

public class ValueAdderCommunication extends PeerCommunication implements
		IChannelManager, IMessageSender {

	protected IChannelInterface channelInterface;

	public ValueAdderCommunication(IPeer peer) {

		assert TCFManager.getTaskManager().isDispatchThread();

		this.peer = peer;

		String[] types = new String[0];

		valueAdderCom = null;

		types = peer.getServiceValueAdderTypes();
		for (String type : types) {
			if (TCFManager.getServiceValueAdderManagers().get(type) != null) {
				for (IServiceValueAdderManager manager : TCFManager
						.getServiceValueAdderManagers().get(type)) {
					manager.createServiceValueAdder(peer, this, this);
				}
			}
		}

		TCFManager.getChannelManagers().get(peer.getChannelType())
				.createChannel(peer, this);
	}

	/* UTILITIES */

	public void terminate() {

		String[] types = new String[0];
		types = peer.getServiceValueAdderTypes();
		for (String type : types) {
			if (TCFManager.getServiceValueAdderManagers().get(type) != null) {
				for (IServiceValueAdderManager manager : TCFManager
						.getServiceValueAdderManagers().get(type)) {
					manager.removeServiceValueAdder(peer);
				}
			}
		}

		TCFManager.getChannelManagers().get(peer.getChannelType())
				.removeChannel(peer);

		valueAdderCom = null;
		peer = null;
	}

	/* ROUTING */

	protected void handleMessageFromTarget(final IMessage msg) {
		assert TCFManager.getTaskManager().isDispatchThread();
		switch (msg.getType()) {
		case Command.TYPE:
			break;
		case Event.TYPE:
			IEventListener eListener = eventListeners.get(msg.getService());
			if (eListener == null)
				break;
			eListener.event(msg.getName(), msg.getData());
			return;
		case Result.TYPE:
			ICommandListener cRListener = commandListeners.remove(msg
					.getToken());
			if (cRListener == null)
				break;
			cRListener.result(msg.getToken(), msg.getData());
			return;
		case Progress.TYPE:
			ICommandListener cPListener = commandListeners.get(msg.getToken());
			if (cPListener == null)
				break;
			cPListener.progress(msg.getToken(), msg.getData());
			return;
		default:
			break;
		}
		handleMessageToServer(msg);
	}

	protected void handleMessageToTarget(final IMessage msg) {
		assert TCFManager.getTaskManager().isDispatchThread();
		for (IMessageSender sender : messageSenders) {
			sender.sendMessage(msg);
		}
	}

	protected void handleMessageFromServer(final IMessage msg) {
		assert TCFManager.getTaskManager().isDispatchThread();
		switch (msg.getType()) {
		case Command.TYPE:
			ICommandServer cServer = commandServers.get(msg.getService());
			if (cServer == null)
				break;
			cServer.command(msg.getToken(), msg.getName(), msg.getData());
			return;
		case Event.TYPE:
			break;
		case Result.TYPE:
			break;
		case Progress.TYPE:
			break;
		default:
			break;
		}
		handleMessageToTarget(msg);
	}

	protected void handleMessageToServer(final IMessage msg) {
		assert TCFManager.getTaskManager().isDispatchThread();
		channelInterface.handleMessage(msg);
	}

	/* PROXY */

	/* PROVIDER */

	public void sendEvent(final String serviceId, final String name,
			final Object[] data) {
		if (!TCFManager.getTaskManager().isDispatchThread()) {
			TCFManager.getTaskManager().invokeLater(new Runnable() {
				public void run() {
					sendEvent(serviceId, name, data);
				}
			});
		} else {
			String token = generateToken();
			IMessage msg = new Event(token, serviceId, name, data);
			handleMessageToServer(msg);
		}
	}

	public void sendProgress(final String token, final Object[] data) {
		if (!TCFManager.getTaskManager().isDispatchThread()) {
			TCFManager.getTaskManager().invokeLater(new Runnable() {
				public void run() {
					sendProgress(token, data);
				}
			});
		} else {
			IMessage msg = new Progress(token, null, null, data);
			handleMessageToServer(msg);
		}
	}

	public void sendResult(final String token, final Object[] data) {
		if (!TCFManager.getTaskManager().isDispatchThread()) {
			TCFManager.getTaskManager().invokeLater(new Runnable() {
				public void run() {
					sendResult(token, data);
				}
			});
		} else {
			IMessage msg = new Result(token, null, null, data);
			handleMessageToServer(msg);
		}
	}

	/* CHANNEL */

	/* CHANNEL MANAGER */

	public void createChannel(IPeer peer, IChannelInterface channelInterface) {
		this.channelInterface = channelInterface;
		channelInterface.addMessageSender(this);
	}

	public String getType() {
		return null;
	}

	public boolean isConnected(IPeer peer) {
		return false;
	}

	public void removeChannel(IPeer peer) {
		channelInterface.removeMessageSender(this);
		terminate();
	}

	public void setTaskUtilities(ITaskUtilities utilities) {
	}

	public void actAsComServer(Map<String, String> serverConfigurations) {
	}

	/* MESSAGE SENDER */

	public void sendMessage(IMessage message) {
		handleMessageFromServer(message);
	}

}
