package org.eclipse.tm.tcf.user;

import java.util.Collection;
import java.util.Map;

import org.eclipse.tm.tcf.extender.channel.IChannelManager;
import org.eclipse.tm.tcf.extender.peer.IPeer;
import org.eclipse.tm.tcf.extender.service.ITaskUtilities.ReturnRunnable;
import org.eclipse.tm.tcf.extender.service.provider.IServiceProvider;
import org.eclipse.tm.tcf.extender.service.proxy.IServiceProxy;
import org.eclipse.tm.tcf.internal.TCFManager;

/**
 * 
 * TCF provide static methods for the frameworks users
 * 
 */
public class TCF {

	public static final int WAITING_TIME = 0;

	public static IPeer createPeer(final String name, final String type,
			final Map<String, String> attributes) throws Exception {
		if (!TCFManager.getTaskManager().isDispatchThread()) {
			Object value = TCFManager.getTaskManager().invokeAndWaitReturn(
					WAITING_TIME, new ReturnRunnable() {
						public Object runAndReturn() throws Exception {
							return createPeer(name, type, attributes);
						}
					});
			return (IPeer) value;
		} else {
			return TCFManager.getPeerManagers().get(type).createPeer(name,
					attributes);
		}

	}

	public static void removePeer(final IPeer peer) throws Exception {
		if (!TCFManager.getTaskManager().isDispatchThread()) {
			TCFManager.getTaskManager().invokeLater(new Runnable() {
				public void run() {
					try {
						removePeer(peer);
					} catch (Exception e) {
						TCFManager.getTaskManager().error(e);
					}
				}
			});
		} else {
			TCFManager.getCommunicationManager().disconnect(peer);
			TCFManager.getPeerManagers().get(peer.getType()).removePeer(peer);
		}
	}

	@SuppressWarnings("unchecked")
	public static Collection<IPeer> getPeers(final String type)
			throws Exception {
		if (!TCFManager.getTaskManager().isDispatchThread()) {
			Object value = TCFManager.getTaskManager().invokeAndWaitReturn(
					WAITING_TIME, new ReturnRunnable() {
						public Object runAndReturn() throws Exception {
							return getPeers(type);
						}
					});
			return (Collection<IPeer>) value;
		} else {
			return TCFManager.getPeerManagers().get(type).getPeers();
		}
	}

	public static void connect(final IPeer peer) {
		if (!TCFManager.getTaskManager().isDispatchThread()) {
			TCFManager.getTaskManager().invokeLater(new Runnable() {
				public void run() {
					connect(peer);
				}
			});
		} else {
			TCFManager.getCommunicationManager().connect(peer);
		}
	}

	public static void disconnect(final IPeer peer) {
		if (!TCFManager.getTaskManager().isDispatchThread()) {
			TCFManager.getTaskManager().invokeLater(new Runnable() {
				public void run() {
					disconnect(peer);
				}
			});
		} else {
			TCFManager.getCommunicationManager().disconnect(peer);
		}
	}

	public static boolean isConnected(final IPeer peer) throws Exception {
		if (!TCFManager.getTaskManager().isDispatchThread()) {
			Object value = TCFManager.getTaskManager().invokeAndWaitReturn(
					WAITING_TIME, new ReturnRunnable() {
						public Object runAndReturn() throws Exception {
							return isConnected(peer);
						}
					});
			return (Boolean) value;
		} else {
			return TCFManager.getCommunicationManager().isConnected(peer);
		}
	}

	// public static void addTCFListener(final ITCFListener listener){
	// if (!TCFManager.getTaskManager().isDispatchThread()) {
	// TCFManager.getTaskManager().invokeLater(new Runnable() {
	// public void run() {
	// addTCFListener(listener);
	// }
	// });
	// } else {
	// TCFManager.getCommunicationManager().addTCFListener(listener);
	// }
	// }
	//	
	// public static void removeTCFListener(final ITCFListener listener){
	// if (!TCFManager.getTaskManager().isDispatchThread()) {
	// TCFManager.getTaskManager().invokeLater(new Runnable() {
	// public void run() {
	// removeTCFListener(listener);
	// }
	// });
	// } else {
	// TCFManager.getCommunicationManager().removeTCFListener(listener);
	// }
	// }

	interface ITCFListener {
		void peerCreated(IPeer peer);

		void peerConnected(IPeer peer);

		void peerDisconnected(IPeer peer);

		void peerRemoved(String name, String type);
	}

	public static IServiceProxy getServiceProxy(final IPeer peer,
			final String serviceName) throws Exception {
		if (!TCFManager.getTaskManager().isDispatchThread()) {
			Object value = TCFManager.getTaskManager().invokeAndWaitReturn(
					WAITING_TIME, new ReturnRunnable() {
						public Object runAndReturn() throws Exception {
							return getServiceProxy(peer, serviceName);
						}
					});
			return (IServiceProxy) value;
		} else {
			return TCFManager.getCommunicationManager().getServiceProxy(peer,
					serviceName);
		}
	}

	public static IServiceProvider getServiceProvider(final IPeer peer,
			final String serviceName) throws Exception {
		if (!TCFManager.getTaskManager().isDispatchThread()) {
			Object value = TCFManager.getTaskManager().invokeAndWaitReturn(
					WAITING_TIME, new ReturnRunnable() {
						public Object runAndReturn() throws Exception {
							return getServiceProvider(peer, serviceName);
						}
					});
			return (IServiceProvider) value;
		} else {
			return TCFManager.getCommunicationManager().getServiceProvider(
					peer, serviceName);
		}
	}

	@SuppressWarnings("unchecked")
	public static Collection<IServiceProxy> getServiceProxies(final IPeer peer)
			throws Exception {
		if (!TCFManager.getTaskManager().isDispatchThread()) {
			Object value = TCFManager.getTaskManager().invokeAndWaitReturn(
					WAITING_TIME, new ReturnRunnable() {
						public Object runAndReturn() throws Exception {
							return getServiceProxies(peer);
						}
					});
			return (Collection<IServiceProxy>) value;
		} else {
			return TCFManager.getCommunicationManager().getServiceProxies(peer);
		}
	}

	@SuppressWarnings("unchecked")
	public static Collection<IServiceProvider> getServiceProviders(
			final IPeer peer) throws Exception {
		if (!TCFManager.getTaskManager().isDispatchThread()) {
			Object value = TCFManager.getTaskManager().invokeAndWaitReturn(
					WAITING_TIME, new ReturnRunnable() {
						public Object runAndReturn() throws Exception {
							return getServiceProviders(peer);
						}
					});
			return (Collection<IServiceProvider>) value;
		} else {
			return TCFManager.getCommunicationManager().getServiceProviders(
					peer);
		}
	}

	public static void setComAsServer(final String channelType,
			final Map<String, String> serverConfigurations) {
		if (!TCFManager.getTaskManager().isDispatchThread()) {
			TCFManager.getTaskManager().invokeLater(new Runnable() {
				public void run() {
					setComAsServer(channelType, serverConfigurations);
				}
			});
		} else {
			IChannelManager manager = TCFManager.getChannelManagers().get(
					channelType);
			if (manager != null)
				manager.actAsComServer(serverConfigurations);
		}
	}

}
